@@ -175,10 +175,7 @@ auth_callback (SMBCCTX *context,
       backend->user == NULL &&
       backend->domain == NULL)
     {
-      /* Try anon login */
-      strncpy (username_out, "", unmaxlen);
-      strncpy (password_out, "", pwmaxlen);
-      /* Try again if anon login fails */
+      /* Try again if kerberos login + anonymous fallback fails */
       backend->mount_try_again = TRUE;
     }
   else
@@ -495,9 +492,15 @@ do_mount (GVfsBackend *backend,
   smb_context->flags = 0;
 #endif
   
+  /* Initial settings:
+   *   - use Kerberos (always)
+   *   - in case of no username specified, try anonymous login
+   */
   smbc_setOptionUseKerberos (smb_context, 1);
-  smbc_setOptionFallbackAfterKerberos (smb_context, 1);  
-  smbc_setOptionNoAutoAnonymousLogin (smb_context, 1);
+  smbc_setOptionFallbackAfterKerberos (smb_context,
+                                       op_backend->user != NULL);
+  smbc_setOptionNoAutoAnonymousLogin (smb_context,
+                                      op_backend->user != NULL);
 
   
 #if 0
@@ -515,8 +518,33 @@ do_mount (GVfsBackend *backend,
 
   op_backend->smb_context = smb_context;
 
+  /* Set the mountspec according to original uri, no matter whether user changes
+     credentials during mount loop. Nautilus and other gio clients depend
+     on correct mountspec, setting it to real (different) credentials would 
+     lead to G_IO_ERROR_NOT_MOUNTED errors
+   */
+
+  /* Translators: This is "<sharename> on <servername>" and is used as name for an SMB share */
+  display_name = g_strdup_printf (_("%s on %s"), op_backend->share, op_backend->server);
+  g_vfs_backend_set_display_name (backend, display_name);
+  g_free (display_name);
+  g_vfs_backend_set_icon_name (backend, "folder-remote");
+
+  smb_mount_spec = g_mount_spec_new ("smb-share");
+  g_mount_spec_set (smb_mount_spec, "share", op_backend->share);
+  g_mount_spec_set (smb_mount_spec, "server", op_backend->server);
+  if (op_backend->user)
+    g_mount_spec_set (smb_mount_spec, "user", op_backend->user);
+  if (op_backend->domain)
+    g_mount_spec_set (smb_mount_spec, "domain", op_backend->domain);
+
+  g_vfs_backend_set_mount_spec (backend, smb_mount_spec);
+  g_mount_spec_unref (smb_mount_spec);
+
   uri = create_smb_uri (op_backend->server, op_backend->share, NULL);
 
+
+  /*  Samba mount loop  */
   op_backend->mount_source = mount_source;
   op_backend->mount_try = 0;
   op_backend->password_save = G_PASSWORD_SAVE_NEVER;
@@ -531,8 +559,17 @@ do_mount (GVfsBackend *backend,
       if (res == 0 ||
 	  (errno != EACCES && errno != EPERM))
 	break;
-      
-      op_backend->mount_try ++;      
+
+      /* The first round is Kerberos-only.  Only if this fails do we enable
+       * NTLMSSP fallback (turning off anonymous fallback, which we've
+       * already tried and failed with).
+       */
+      if (op_backend->mount_try == 0)
+        {
+          smbc_setOptionFallbackAfterKerberos (op_backend->smb_context, 1);
+          smbc_setOptionNoAutoAnonymousLogin (op_backend->smb_context, 1);
+        }
+      op_backend->mount_try ++;
     }
   while (op_backend->mount_try_again);
   
@@ -553,25 +590,6 @@ do_mount (GVfsBackend *backend,
 
   /* Mount was successful */
 
-  /* Translators: This is "<sharename> on <servername>" and is used as name for an SMB share */
-  display_name = g_strdup_printf (_("%s on %s"), op_backend->share, op_backend->server);
-  g_vfs_backend_set_display_name (backend, display_name);
-  g_free (display_name);
-  g_vfs_backend_set_icon_name (backend, "folder-remote");
-
-  smb_mount_spec = g_mount_spec_new ("smb-share");
-  g_mount_spec_set (smb_mount_spec, "share", op_backend->share);
-  g_mount_spec_set (smb_mount_spec, "server", op_backend->server);
-  if (op_backend->last_user && strlen(op_backend->last_user) > 0)
-    {
-      g_mount_spec_set (smb_mount_spec, "user", op_backend->last_user);
-      if (op_backend->last_domain)
-        g_mount_spec_set (smb_mount_spec, "domain", op_backend->last_domain);
-    }
-
-  g_vfs_backend_set_mount_spec (backend, smb_mount_spec);
-  g_mount_spec_unref (smb_mount_spec);
-
   g_vfs_keyring_save_password (op_backend->last_user,
 			       op_backend->server,
 			       op_backend->last_domain,
@@ -1015,9 +1033,9 @@ do_replace (GVfsBackend *backend,
     {
       int errsv = errno;
 
-      g_set_error (&error, G_IO_ERROR,
-		   g_io_error_from_errno (errsv),
-		   "%s", g_strerror (errsv));
+      g_set_error_literal (&error, G_IO_ERROR,
+			   g_io_error_from_errno (errsv),
+			   g_strerror (errsv));
       goto error;
     }
   else if (file == NULL && errno == EEXIST)
@@ -1032,11 +1050,10 @@ do_replace (GVfsBackend *backend,
 	      if (strcmp (etag, current_etag) != 0)
 		{
 		  g_free (current_etag);
-		  g_set_error (&error,
-			       G_IO_ERROR,
-			       G_IO_ERROR_WRONG_ETAG,
-			       "%s",
-                               _("The file was externally modified"));
+		  g_set_error_literal (&error,
+				       G_IO_ERROR,
+				       G_IO_ERROR_WRONG_ETAG,
+                	               _("The file was externally modified"));
 		  goto error;
 		}
 	      g_free (current_etag);
@@ -1064,17 +1081,15 @@ do_replace (GVfsBackend *backend,
 	      if (!copy_file (op_backend, G_VFS_JOB (job), uri, backup_uri))
 		{
 		  if (g_vfs_job_is_cancelled (G_VFS_JOB (job)))
-		    g_set_error (&error,
-				 G_IO_ERROR,
-				 G_IO_ERROR_CANCELLED,
-                                 "%s",
-				 _("Operation was cancelled"));
+		    g_set_error_literal (&error,
+					 G_IO_ERROR,
+					 G_IO_ERROR_CANCELLED,
+					 _("Operation was cancelled"));
 		  else
-		    g_set_error (&error,
-				 G_IO_ERROR,
-				 G_IO_ERROR_CANT_CREATE_BACKUP,
-                                 "%s",
-				 _("Backup file creation failed"));
+		    g_set_error_literal (&error,
+					 G_IO_ERROR,
+					 G_IO_ERROR_CANT_CREATE_BACKUP,
+					 _("Backup file creation failed"));
 		  goto error;
 		}
 	      g_free (backup_uri);
@@ -1087,9 +1102,9 @@ do_replace (GVfsBackend *backend,
 	    {
               int errsv = errno;
 
-	      g_set_error (&error, G_IO_ERROR,
-			   g_io_error_from_errno (errsv),
-			   "%s", g_strerror (errsv));
+	      g_set_error_literal (&error, G_IO_ERROR,
+				   g_io_error_from_errno (errsv),
+				   g_strerror (errsv));
 	      goto error;
 	    }
 	}
@@ -1540,9 +1555,9 @@ do_enumerate (GVfsBackend *backend,
       int errsv = errno;
 
       error = NULL;
-      g_set_error (&error, G_IO_ERROR,
-		   g_io_error_from_errno (errsv),
-		   "%s", g_strerror (errsv));
+      g_set_error_literal (&error, G_IO_ERROR,
+			   g_io_error_from_errno (errsv),
+			   g_strerror (errsv));
       goto error;
     }
 
