@@ -1,5 +1,5 @@
 /* GIO - GLib Input, Output and Streaming Library
- * 
+ *
  * Copyright (C) 2006-2007 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
@@ -140,6 +140,25 @@ changed_in_idle (gpointer data)
 #define MEGABYTE_FACTOR (1000.0 * 1000.0)
 #define GIGABYTE_FACTOR (1000.0 * 1000.0 * 1000.0)
 
+/**
+ * format_size_for_display:
+ * @size: a number of octects
+ *
+ * Format a human readable string that can conveys how much storage a
+ * user-visible drive or piece of media can hold.
+ *
+ * As a matter of policy, we want this string to resemble what's on
+ * the packaging of the drive/media. Since all manufacturers use
+ * powers of 10, g_format_size_for_display() is not suitable here.
+ *
+ * TODO: we probably want to round to nearest power of two if @size is
+ * "close" (e.g. within 5%) - this is to avoid e.g. 63.4G when the
+ * packaging says "64G drive". We could also use per-drive or
+ * per-media quirks to make a better guess.
+ *
+ * Returns: A human readable string, caller must free it using
+ * g_free().
+ **/
 static char *
 format_size_for_display (guint64 size)
 {
@@ -165,36 +184,6 @@ format_size_for_display (guint64 size)
   return str;
 }
 
-static char **
-dupv_and_uniqify (char **str_array)
-{
-  int n, m, o;
-  int len;
-  char **result;
-
-  result = g_strdupv (str_array);
-  len = g_strv_length (result);
-
-  for (n = 0; n < len; n++)
-    {
-      char *s = result[n];
-      for (m = n + 1; m < len; m++)
-        {
-          char *p = result[m];
-          if (strcmp (s, p) == 0)
-            {
-              for (o = m + 1; o < len; o++)
-                result[o - 1] = result[o];
-              len--;
-              result[len] = NULL;
-              m--;
-            }
-        }
-    }
-
-  return result;
-}
-
 static void
 do_update_from_hal (GHalVolume *mv)
 {
@@ -828,14 +817,11 @@ spawn_do (GVolume             *volume,
                       &child_pid,
                       &error))
     {
-      GSimpleAsyncResult *simple;
-      simple = g_simple_async_result_new_from_error (data->object,
-                                                     data->callback,
-                                                     data->user_data,
-                                                     error);
+      g_simple_async_report_gerror_in_idle (data->object,
+                                            data->callback,
+                                            data->user_data,
+                                            error);
       g_object_unref (data->object);
-      g_simple_async_result_complete (simple);
-      g_object_unref (simple);
       g_error_free (error);
       g_free (data);
       return;
@@ -858,6 +844,7 @@ mount_foreign_callback (GObject *source_object,
 {
   ForeignMountOp *data = user_data;
   data->callback (G_OBJECT (data->enclosing_volume), res, data->user_data);
+  g_object_unref (data->enclosing_volume);
   g_free (data);
 }
 
@@ -882,7 +869,7 @@ g_hal_volume_mount (GVolume             *volume,
       ForeignMountOp *data;
 
       data = g_new0 (ForeignMountOp, 1);
-      data->enclosing_volume = hal_volume;
+      data->enclosing_volume = g_object_ref (hal_volume);
       data->callback = callback;
       data->user_data = user_data;
 
@@ -937,6 +924,7 @@ eject_wrapper_callback (GObject *source_object,
 {
   EjectWrapperOp *data  = user_data;
   data->callback (data->object, res, data->user_data);
+  g_object_unref (data->object);
   g_free (data);
 }
 
@@ -957,7 +945,7 @@ g_hal_volume_eject (GVolume              *volume,
   if (hal_volume->drive != NULL)
     drive = g_object_ref (hal_volume->drive);
   G_UNLOCK (hal_volume);
-  
+
   if (drive != NULL)
     {
       EjectWrapperOp *data;
@@ -1016,7 +1004,7 @@ g_hal_volume_enumerate_identifiers (GVolume *volume)
   const char *label, *uuid;
 
   G_LOCK (hal_volume);
-  
+
   res = g_ptr_array_new ();
 
   g_ptr_array_add (res,
@@ -1025,10 +1013,10 @@ g_hal_volume_enumerate_identifiers (GVolume *volume)
   if (hal_volume->device_path && *hal_volume->device_path != 0)
     g_ptr_array_add (res,
                      g_strdup (G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE));
-    
+
   label = hal_device_get_property_string (hal_volume->device, "volume.label");
   uuid = hal_device_get_property_string (hal_volume->device, "volume.uuid");
-  
+
   if (label && *label != 0)
     g_ptr_array_add (res,
                      g_strdup (G_VOLUME_IDENTIFIER_KIND_LABEL));
@@ -1041,7 +1029,7 @@ g_hal_volume_enumerate_identifiers (GVolume *volume)
   g_ptr_array_add (res, NULL);
 
   G_UNLOCK (hal_volume);
-  
+
   return (char **)g_ptr_array_free (res, FALSE);
 }
 
@@ -1049,8 +1037,14 @@ static GFile *
 g_hal_volume_get_activation_root (GVolume *volume)
 {
   GHalVolume *hal_volume = G_HAL_VOLUME (volume);
+  GFile *root = NULL;
 
-  return hal_volume->foreign_mount_root != NULL ? g_object_ref (hal_volume->foreign_mount_root) : NULL;
+  G_LOCK (hal_volume);
+  if (hal_volume->foreign_mount_root != NULL)
+    root = g_object_ref (hal_volume->foreign_mount_root);
+  G_UNLOCK (hal_volume);
+
+  return root;
 }
 
 static void
